# an editor that uses rich text prompts to edit time entries
# edit command will prompt for each field with default values from the entry
# steps through each field and allows editing
# default field order list?
# how should the editor handle tags?
